@@ -36,7 +36,7 @@
 #' so that \code{latexmk()} knows where to find executables like
 #' \command{pdflatex}. For example, if you are using Windows and your TinyTeX is
 #' on an external drive \file{Z:/} under the folder \file{TinyTeX}, you may set
-#' \code{options(tinytex.tlmgr.path = "Z:/TinyTeX/bin/win32/tlmgr.bat")}.
+#' \code{options(tinytex.tlmgr.path = "Z:/TinyTeX/bin/windows/tlmgr.bat")}.
 #' Usually you should not need to set this option because TinyTeX can add itself
 #' to the \code{PATH} variable during installation or via
 #' \code{\link{use_tinytex}()}. In case both methods fail, you can use this
@@ -62,7 +62,8 @@
 #'   \code{options(tinytex.install_packages = FALSE)}.
 #' @param pdf_file Path to the PDF output file. By default, it is under the same
 #'   directory as the input \code{file} and also has the same base name. When
-#'   \code{engine == 'latex'}, this will be a DVI file.
+#'   \code{engine == 'latex'} or \code{engine_args} contains \verb{--no-pdf} or
+#'   \code{--output-format=dvi}, this will be a DVI file.
 #' @param clean Whether to clean up auxiliary files after compilation (can be
 #'   set in the global option \code{tinytex.clean}, which defaults to
 #'   \code{TRUE}).
@@ -73,7 +74,7 @@ latexmk = function(
   file, engine = c('pdflatex', 'xelatex', 'lualatex', 'latex', 'tectonic'),
   bib_engine = c('bibtex', 'biber'), engine_args = NULL, emulation = TRUE,
   min_times = 1, max_times = 10, install_packages = emulation && tlmgr_writable(),
-  pdf_file = gsub('tex$', 'pdf', file), clean = TRUE
+  pdf_file = NULL, clean = TRUE
 ) {
   if (!grepl('[.]tex$', file))
     stop("The input file '", file, "' does not have the .tex extension")
@@ -81,7 +82,9 @@ latexmk = function(
   if (missing(engine)) engine = getOption('tinytex.engine', engine)
   engine = gsub('^(pdf|xe|lua)(tex)$', '\\1la\\2', engine)  # normalize *tex to *latex
   engine = match.arg(engine)
-  is_latex = engine == 'latex'
+  is_dvi = engine == 'latex' ||
+    any(grepl('(^| )(--output-format=dvi|--no-pdf)( |$)', engine_args))
+  ext = if (is_dvi) { if (engine == 'xelatex') 'xdv' else 'dvi' } else 'pdf'
   tweak_path()
   if (missing(emulation)) emulation = getOption('tinytex.latexmk.emulation', emulation)
   if (!emulation) {
@@ -100,20 +103,19 @@ latexmk = function(
   if (missing(bib_engine)) bib_engine = getOption('tinytex.bib_engine', bib_engine)
   if (missing(engine_args)) engine_args = getOption('tinytex.engine_args', engine_args)
   if (missing(clean)) clean = getOption('tinytex.clean', TRUE)
-  pdf = gsub('tex$', if (is_latex) 'dvi' else 'pdf', basename(file))
+  out = with_ext(basename(file), ext)
   if (!is.null(output_dir <- getOption('tinytex.output_dir'))) {
     output_dir_arg = shQuote(paste0(if (emulation) '-', '-output-directory=', output_dir))
     if (length(grep(output_dir_arg, engine_args, fixed = TRUE)) == 0) stop(
       "When you set the global option 'tinytex.output_dir', the argument 'engine_args' ",
       "must contain this value: ", capture.output(dput(output_dir_arg))
     )
-    pdf = file.path(output_dir, pdf)
-    if (missing(pdf_file)) pdf_file = file.path(output_dir, basename(pdf_file))
+    if (is.null(pdf_file)) out = file.path(output_dir, out)
   }
-  if (is_latex) pdf_file = with_ext(pdf_file, 'dvi')
+  if (is.null(pdf_file)) pdf_file = with_ext(file, ext)
   check_pdf = function() {
-    if (!file.exists(pdf)) show_latex_error(file, with_ext(pdf, 'log'), TRUE)
-    file_rename(pdf, pdf_file)
+    if (!file.exists(out)) show_latex_error(file, with_ext(out, 'log'), TRUE)
+    xfun::file_rename(out, pdf_file)
     pdf_file
   }
   if (engine == 'tectonic') {
@@ -129,7 +131,7 @@ latexmk = function(
   }
   system2_quiet('latexmk', c(
     '-latexoption=-halt-on-error -interaction=batchmode',
-    if (is_latex) '-latex=latex' else paste0('-pdf -pdflatex=', engine),
+    if (is_dvi) '-latex=latex' else paste0('-pdf -pdflatex=', engine),
     engine_args, shQuote(file)
   ), error = {
     if (install_packages) warning(
@@ -248,7 +250,7 @@ latexmk_emu = function(
       build_bib = function() system2_quiet(bib_engine, shQuote(aux), error = {
         check_blg = function() {
           if (!file.exists(blg)) return(TRUE)
-          x = readLines(blg)
+          x = read_lines(blg)
           if (length(grep('error message', x)) == 0) return(TRUE)
           warn = function() {
             warning(
@@ -282,7 +284,7 @@ latexmk_emu = function(
 }
 
 require_bibtex = function(aux) {
-  x = readLines(aux)
+  x = read_lines(aux)
   r = length(grep('^\\\\citation\\{', x)) && length(grep('^\\\\bibdata\\{', x)) &&
     length(grep('^\\\\bibstyle\\{', x))
   if (r && !tlmgr_available() && os == 'windows') tweak_aux(aux, x)
@@ -291,14 +293,14 @@ require_bibtex = function(aux) {
 
 # remove the .bib extension in \bibdata{} in the .aux file, because bibtex on
 # Windows requires no .bib extension (sigh)
-tweak_aux = function(aux, x = readLines(aux)) {
+tweak_aux = function(aux, x = read_lines(aux)) {
   r = '^\\\\bibdata\\{.+\\}\\s*$'
   if (length(i <- grep(r, x)) == 0) return()
   x[i] = gsub('[.]bib([,}])', '\\1', x[i])
   writeLines(x, aux)
 }
 
-needs_rerun = function(log, text = xfun::read_utf8(log)) {
+needs_rerun = function(log, text = read_lines(log)) {
   any(grepl(
     '(Rerun to get |Please \\(?re\\)?run | Rerun LaTeX\\.)', text,
     useBytes = TRUE
@@ -314,7 +316,7 @@ system2_quiet = function(..., error = NULL, logfile = NULL, fail_rerun = TRUE) {
 
   # run the command quietly if possible
   res = system2(..., stdout = if (use_file_stdout()) f1 else FALSE, stderr = f2)
-  if (is.character(logfile) && file.exists(f2) && length(e <- xfun::read_utf8(f2))) {
+  if (is.character(logfile) && file.exists(f2) && length(e <- read_lines(f2))) {
     i = grep('^\\s*$', e, invert = TRUE)
     e[i] = paste('!', e[i])  # prepend ! to non-empty error messages
     cat('', e, file = logfile, sep = '\n', append = TRUE)
@@ -339,7 +341,7 @@ show_latex_error = function(
 ) {
   e = c('LaTeX failed to compile ', file, '. See https://yihui.org/tinytex/r/#debugging for debugging tips.')
   if (!file.exists(logfile)) stop(e, call. = FALSE)
-  x = readLines(logfile, warn = FALSE)
+  x = read_lines(logfile)
   b = grep('^\\s*$', x)  # blank lines
   b = c(b, which(x == "Here is how much of TeX's memory you used:"))
   m = NULL
@@ -376,7 +378,7 @@ check_inline_math = function(x, f) {
 }
 
 check_unicode = function(x) {
-  if (length(grep('! Package inputenc Error: Unicode character', x))) message(
+  if (length(grep('! (Package inputenc|LaTeX) Error: Unicode character', x))) message(
     'Try other LaTeX engines instead (e.g., xelatex) if you are using pdflatex.',
     if ('rmarkdown' %in% loadedNamespaces())
       ' See https://bookdown.org/yihui/rmarkdown-cookbook/latex-unicode.html'
@@ -388,7 +390,7 @@ latex_warning = function(file, show = getOption('tinytex.latexmk.warning', TRUE)
   if (!file.exists(file)) return()
   # if the option tinytex.latexmk.warning = FALSE, delete the log in latexmk_emu()
   if (!show && missing(show)) return()
-  x = readLines(file, warn = FALSE)
+  x = read_lines(file)
   if (length(i <- grep('^(LaTeX|Package [[:alnum:]]+) Warning:', x)) == 0) return()
   # these warnings may be okay (our Pandoc LaTeX template in rmarkdown may need an update)
   i = i[grep('^Package (fixltx2e|caption|hyperref) Warning:', x[i], invert = TRUE)]
@@ -421,6 +423,7 @@ check_babel = function(text) {
   if (length(m <- grep_sub(r, 'hyphen-\\2', text)) == 0) return(FALSE)
   # (babel) the language `German (new orthography)' into the format
   m = gsub('\\s.*', '', m)
+  m[m == 'hyphen-pinyin'] = 'hyphen-chinese'
   tlmgr_install(tolower(m)) == 0
 }
 
@@ -473,14 +476,6 @@ exist_files = function(files) {
   files[utils::file_test('-f', files)]
 }
 
-# use file.copy() if file.rename() fails
-file_rename = function(from, to) {
-  if (from == to) return(TRUE)
-  if (!suppressWarnings(file.rename(from, to))) {
-    if (file.copy(from, to, overwrite = TRUE)) file.remove(from)
-  }
-}
-
 #' Find missing LaTeX packages from a LaTeX log file
 #'
 #' Analyze the error messages in a LaTeX log file to figure out the names of
@@ -502,7 +497,7 @@ file_rename = function(from, to) {
 #' @return A character vector of LaTeX package names.
 #' @export
 parse_packages = function(
-  log, text = readLines(log), files = detect_files(text), quiet = rep(FALSE, 3)
+  log, text = read_lines(log), files = detect_files(text), quiet = rep(FALSE, 3)
 ) {
   pkgs = character(); quiet = rep_len(quiet, length.out = 3)
   x = unique(c(files, miss_font()))
@@ -560,6 +555,7 @@ regex_errors = function() {
   # ! I can't find file `hyph-de-1901.ec.tex'.
   # ! Package pdfx Error: No color profile sRGB_IEC61966-2-1_black_scaled.icc found
   # No file LGRcmr.fd. ! LaTeX Error: This NFSS system isn't set up properly.
+  # (fontspec)                The font "LibertinusSerif-Regular" cannot be
   list(
     font = c(
       # error messages about missing fonts (don't move the first item below, as
@@ -568,7 +564,8 @@ regex_errors = function() {
       ".*! Font [^=]+=([^ ]+).+ not loadable.*",
       '.*! .*The font "([^"]+)" cannot be found.*',
       '.*!.+ error:.+\\(file ([^)]+)\\): .*',
-      '.*Unable to find TFM file "([^"]+)".*'
+      '.*Unable to find TFM file "([^"]+)".*',
+      '.*\\(fontspec\\)\\s+The font "([^"]+)" cannot be.*'
     ),
     fd = c(
       # font definition files
@@ -589,6 +586,10 @@ regex_errors = function() {
       # when a required tikz library is missing
       '.* (tikzlibrary[^ ]+?[.]code[.]tex).*'
     ),
+    l3backend = c(
+      # L3 programming layer mismatch (#424)
+      '^File: ([^ ]+) \\d{4,}-\\d{2}-\\d{2} .*$'
+    ),
     style = c(
       # missing .sty or commands
       ".* Loading '([^']+)' aborted!",
@@ -613,6 +614,11 @@ detect_files = function(text) {
     v = grep_sub(p, '\\1', x)
     if (length(v) == 0) return(v)
     if (p == r$tikz && length(grep('! Package tikz Error:', text)) == 0) return()
+    # if the problem is caused by the L3 programming layer mismatch, use the
+    # last found file before the error line, which should be from l3backend
+    if (p == r$l3backend) return(
+      if (length(grep('^! Undefined control sequence', text)) > 0) tail(v, 1)
+    )
     # these are some known filenames
     for (i in c('epstopdf', grep('[.]', names(r), value = TRUE))) {
       if (p %in% r[[i]]) return(i)
@@ -639,7 +645,7 @@ parse_install = function(...) {
 miss_font = function() {
   if (!file.exists(f <- 'missfont.log')) return()
   on.exit(unlink(f), add = TRUE)
-  x = gsub('\\s*$', '', readLines(f))
+  x = gsub('\\s*$', '', read_lines(f))
   x = grep('.+\\s+.+', x, value = TRUE)
   if (length(x) == 0) return()
   x1 = gsub('.+\\s+', '', x)  # possibly missing fonts
@@ -661,12 +667,12 @@ texhash = function() {
 
 updmap = function(usermode = FALSE) {
   tweak_path()
-  system2(if (usermode) 'updmap-user' else 'updmap-sys')
+  system2('updmap', if (usermode) '--user' else '--sys')
 }
 
 fmtutil = function(usermode = FALSE, ...) {
   tweak_path()
-  system2(if (usermode) 'fmtutil-user' else 'fmtutil-sys', '--all', ...)
+  system2('fmtutil', c(if (usermode) '--user' else '--sys', '--all'), ...)
 }
 
 fc_cache = function(args = c('-v', '-r')) {
@@ -682,7 +688,7 @@ refresh_all = function(...) {
 }
 
 # look up files in the Kpathsea library, e.g., kpsewhich('Sweave.sty')
-kpsewhich = function(filename, options = character()) {
+kpsewhich = function(filename, options = character(), ...) {
   tweak_path()
-  system2('kpsewhich', c(options, shQuote(filename)))
+  system2('kpsewhich', c(options, shQuote(filename)), ...)
 }
