@@ -142,7 +142,7 @@ need_add_path = function() {
 
 is_writable = function(p) file.access(p, 2) == 0
 
-tlmgr_writable = function() is_writable(Sys.which('tlmgr'))
+tlmgr_writable = function() is_tinytex() && is_writable(Sys.which('tlmgr'))
 
 #' Check if certain LaTeX packages are installed
 #'
@@ -376,15 +376,6 @@ tl_platforms = function(print = FALSE) {
   } else x
 }
 
-# a copy of the returned result from tl_platform() is saved here because
-# tl_platform() is a little slow and requires Internet connection
-.tl_platforms = c(
-  'aarch64-linux', 'amd64-freebsd', 'amd64-netbsd', 'armhf-linux',
-  'i386-freebsd', 'i386-linux', 'i386-netbsd', 'i386-solaris', 'universal-darwin',
-  'windows', 'x86_64-cygwin', 'x86_64-darwinlegacy', 'x86_64-linux',
-  'x86_64-linuxmusl', 'x86_64-solaris'
-)
-
 # remove the platform suffixes from texlive package names, and optionally keep
 # the suffixes for certain platforms
 tl_names = function(x, platform = tl_platform()) {
